import { defineStore } from "pinia";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";
import type {
  EquipmentViewModel,
  CreateEquipmentViewModel,
  UpdateEquipmentViewModel,
} from "../../../viewmodels/admin/configuration/equipment.models";

export const useEquipmentStore = defineStore("equipment", {
  state: () => {
    return {
      equipments: [] as Array<EquipmentViewModel & { tab_pos: number }>,
      availableEquipments: [] as Array<EquipmentViewModel>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchEquipments(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.equipments = [];
      this.loading = "loading";
      http
        .get(`/admin/equipment?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.equipments
            .filter((elem: EquipmentViewModel) => this.equipments.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: EquipmentViewModel, index: number): EquipmentViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: EquipmentViewModel & { tab_pos: number }) => {
              this.equipments.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    getAvailableEquipments(missionId: string) {
      this.availableEquipments = [];
      http
        .get(`/admin/equipment/available/${missionId}`)
        .then((res) => {
          this.availableEquipments = res.data;
        })
        .catch((err) => {});
    },
    async getAllEquipments(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/equipment?noLimit=true`).then((res) => {
        return { ...res, data: res.data.equipments };
      });
    },
    async getEquipmentsByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/equipment/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.equipments };
        });
    },
    async searchEquipments(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/equipment?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.equipments };
      });
    },
    fetchEquipmentById(id: string) {
      return http.get(`/admin/equipment/${id}`);
    },
    async createEquipment(equipment: CreateEquipmentViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/equipment`, equipment);
      this.fetchEquipments(0, 25, "", true);
      return result;
    },
    async updateEquipment(equipment: UpdateEquipmentViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/equipment/${equipment.id}`, equipment);
      this.fetchEquipments(0, 25, "", true);
      return result;
    },
    async decommissionEquipment(equipmentId: string) {
      await http.patch(`/admin/equipment/${equipmentId}/decommission`).catch(() => {});
      this.fetchEquipments(0, 25, "", true);
    },
    async deleteEquipment(equipment: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/equipment/${equipment}`);
      this.fetchEquipments(0, 25, "", true);
      return result;
    },
  },
});
