import { defineStore } from "pinia";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";
import type {
  VehicleViewModel,
  CreateVehicleViewModel,
  UpdateVehicleViewModel,
} from "../../../viewmodels/admin/configuration/vehicle.models";

export const useVehicleStore = defineStore("vehicle", {
  state: () => {
    return {
      vehicles: [] as Array<VehicleViewModel & { tab_pos: number }>,
      availableVehicles: [] as Array<VehicleViewModel>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchVehicles(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.vehicles = [];
      this.loading = "loading";
      http
        .get(`/admin/vehicle?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.vehicles
            .filter((elem: VehicleViewModel) => this.vehicles.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: VehicleViewModel, index: number): VehicleViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: VehicleViewModel & { tab_pos: number }) => {
              this.vehicles.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    getAvailableVehicles(missionId: string) {
      this.availableVehicles = [];
      http
        .get(`/admin/vehicle/available/${missionId}`)
        .then((res) => {
          this.availableVehicles = res.data;
        })
        .catch((err) => {});
    },
    async getAllVehicles(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/vehicle?noLimit=true`).then((res) => {
        return { ...res, data: res.data.vehicles };
      });
    },
    async getVehiclesByIds(ids: Array<string>): Promise<AxiosResponse<any, any>> {
      return await http
        .post(`/admin/vehicle/ids`, {
          ids,
        })
        .then((res) => {
          return { ...res, data: res.data.vehicles };
        });
    },
    async searchVehicles(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/vehicle?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.vehicles };
      });
    },
    fetchVehicleById(id: string) {
      return http.get(`/admin/vehicle/${id}`);
    },
    async createVehicle(vehicle: CreateVehicleViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/vehicle`, vehicle);
      this.fetchVehicles(0, 25, "", true);
      return result;
    },
    async updateVehicle(vehicle: UpdateVehicleViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/vehicle/${vehicle.id}`, vehicle);
      this.fetchVehicles(0, 25, "", true);
      return result;
    },
    async decommissionVehicle(vehicleId: string) {
      await http.patch(`/admin/vehicle/${vehicleId}/decommission`).catch(() => {});
      this.fetchVehicles(0, 25, "", true);
    },
    async deleteVehicle(vehicle: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/vehicle/${vehicle}`);
      this.fetchVehicles(0, 25, "", true);
      return result;
    },
  },
});
