export type PermissionSection = "operation" | "configuration" | "management";

export type PermissionModule = "mission" | "force" | "vehicle" | "equipment" | "user" | "role" | "backup" | "import";

export type PermissionType = "read" | "create" | "update" | "delete";

export type PermissionString =
  | `${PermissionSection}.${PermissionModule}.${PermissionType}` // für spezifische Berechtigungen
  | `${PermissionSection}.${PermissionModule}.*` // für alle Berechtigungen in einem Modul
  | `${PermissionSection}.${PermissionType}` // für spezifische Berechtigungen in einem Abschnitt
  | `${PermissionSection}.*` // für alle Berechtigungen in einem Abschnitt
  | `additional.${string}.${string}` // additional
  | "*"; // für Admin

export type PermissionObject = {
  [section in PermissionSection]?: {
    [module in PermissionModule]?: Array<PermissionType> | "*";
  } & { all?: Array<PermissionType> | "*" };
} & {
  admin?: boolean;
} & {
  additional?: { [key: string]: string };
};

export type SectionsAndModulesObject = {
  [section in PermissionSection]: Array<PermissionModule>;
} & {
  additional?: Array<{
    key: string;
    name: string;
    type: "number" | "string";
    emptyIfAdmin: boolean;
  }>;
};

export const permissionSections: Array<PermissionSection> = ["operation", "configuration", "management"];
export const permissionModules: Array<PermissionModule> = [
  "mission",
  "force",
  "vehicle",
  "equipment",
  "user",
  "role",
  "backup",
  "import",
];
export const permissionTypes: Array<PermissionType> = ["read", "create", "update", "delete"];
export const sectionsAndModules: SectionsAndModulesObject = {
  operation: ["mission"],
  configuration: ["force", "vehicle", "equipment"],
  management: ["user", "role", "backup", "import"],
  additional: [
    { key: "maxVisInDays", name: "max Sichtbarkeit in Tagen", type: "number", emptyIfAdmin: true },
    { key: "maxVisByMissions", name: "max Sichtbarkeit an Einsätzen", type: "number", emptyIfAdmin: true },
  ],
};
